/*
** $Id: mod_lua.c,v 1.1.1.1 2003/06/30 19:18:24 michel Exp $
** mod_lua - Apache module for Lua language
** See copyright notice in distro's COPYRIGHT file
*/

#include <httpd.h>
#include <http_config.h>

#include "mod_lua.h"
#include "modlua_config.h"
#include "modlua_lua.h"
#include "modlua_handler.h"

static void modlua_init(server_rec *s, pool *p)
{
    /* Likely modlua_lua_init already called in configuration process, but
     * we call here to prevent when this didn't make. */
    modlua_lua_init(p);
}

/* ====================================== */
#define EVENT(diretive, event) {"" diretive, \
    modlua_cmd_handler, (void *)event, RSRC_CONF|ACCESS_CONF|OR_NONE, \
    ITERATE, "" diretive}

static const command_rec modlua_cmds[] =
{
    {"LuaPath", modlua_cmd_path, NULL, RSRC_CONF, TAKE1, "LuaPath"},    
    {"LuaRequire", modlua_cmd_require, NULL, RSRC_CONF, ITERATE, "LuaRequire"},
    EVENT("LuaHandlerPostReadRequest",  EVENT_POSTREADREQUEST),
    EVENT("LuaHandlerTrans",            EVENT_TRANS),
    EVENT("LuaHandlerHeaderParser",     EVENT_HEADERPARSER),
    EVENT("LuaHandlerAccess",           EVENT_ACCESS),
    EVENT("LuaHandlerAuthen",           EVENT_AUTHEN),
    EVENT("LuaHandlerAuthz",            EVENT_AUTHZ),
    EVENT("LuaHandlerType",             EVENT_TYPE),
    EVENT("LuaHandlerFixup",            EVENT_FIXUP),
    EVENT("LuaHandlerResponse",         EVENT_RESPONSE),
    EVENT("LuaHandlerLog",              EVENT_LOG),
    {NULL}
};

/* ====================================== */
static const handler_rec modlua_handlers[] =
{
    {"lua-script", modlua_handler_response},
    {NULL}
};

/* ====================================== */
module MODULE_VAR_EXPORT lua_module =
{
    STANDARD_MODULE_STUFF,
    modlua_init,                    /* module initializer                   */
    modlua_config_dir_create,       /* per-directory config creator         */
    modlua_config_dir_merge,        /* dir config merger                    */
    modlua_config_srv_create,       /* server config creator                */
    modlua_config_srv_merge,        /* server config merger                 */
    modlua_cmds,                    /* config diretive table                */
    modlua_handlers,                /* [9]  content handlers                */
    modlua_handler_trans,           /* [2]  URI-to-filename translation     */
    modlua_handler_authen,          /* [5]  check/validate user_id          */
    modlua_handler_authz,           /* [6]  check user_id is valid *here*   */
    modlua_handler_access,          /* [4]  check access by host address    */
    modlua_handler_type,            /* [7]  MIME type checker/setter        */
    modlua_handler_fixup,           /* [8]  fixups                          */
    modlua_handler_log,             /* [10] logger                          */
    modlua_handler_headerparser,    /* [3]  header parser                   */
    NULL,                           /* process initialization               */
    NULL,                           /* process exit/cleanup                 */
    modlua_handler_postreadrequest  /* [1]  post read_request handling      */
};
