/*
** $Id: modlua_config.c,v 1.1.1.1 2003/06/30 19:18:24 michel Exp $
** mod_lua - Apache module for Lua language
** See copyright notice in distro's COPYRIGHT file
*/

#include <httpd.h>

#include "modlua_config.h"
#include "modlua_lua.h"
#include "mod_lua.h"

void *modlua_config_srv_create(pool *p, server_rec *s)
{
    modlua_config_srv_t *scfg = ap_palloc(p, sizeof(modlua_config_srv_t));
    scfg->path = "";
    return (void *)scfg;
}

void *modlua_config_srv_merge(pool *p, void *basev, void *addv)
{
    modlua_config_srv_t *mrg  = ap_palloc(p, sizeof(modlua_config_srv_t));
    modlua_config_srv_t *base = (modlua_config_srv_t *)basev;
    modlua_config_srv_t *add  = (modlua_config_srv_t *)addv;
    
    mrg->path = *(add->path) ? add->path : base->path;

    return (void *)mrg;
}

void *modlua_config_dir_create(pool *p, char *dir)
{
    int i;
    modlua_config_dir_t *dcfg = ap_palloc(p, sizeof(modlua_config_dir_t));
    for (i = EVENT_FIRST; i <= EVENT_LAST; i++) {
        dcfg->handlers[i] = ap_make_array(p, 0, sizeof(char *));
    }
    return (void *)dcfg;
}

void *modlua_config_dir_merge(pool *p, void *basev, void *addv)
{
    int i;
    modlua_config_dir_t *mrg  = ap_palloc(p, sizeof(modlua_config_dir_t));
    modlua_config_dir_t *base = (modlua_config_dir_t *)basev;
    modlua_config_dir_t *add  = (modlua_config_dir_t *)addv;

    for (i = EVENT_FIRST; i <= EVENT_LAST; i++) {
        mrg->handlers[i] = add->handlers[i]->nelts ?
                           add->handlers[i] :
                           base->handlers[i];
    }

    return (void *)mrg;
}

const char *modlua_cmd_path(cmd_parms *parms, void *mconfig, const char *arg)
{
    modlua_config_srv_t *scfg = (modlua_config_srv_t *)
        ap_get_module_config(parms->server->module_config, &lua_module);
    char *path = "";
    char *sub_path;

    modlua_lua_init(parms->pool);
    
    /* We set path with absolute paths. */
    while (*arg && (sub_path = ap_getword(parms->temp_pool, &arg, ';'))) {
        ap_no2slash(sub_path);
        sub_path = ap_server_root_relative(parms->temp_pool, sub_path);
        if (*path) {
            path = ap_pstrcat(parms->temp_pool, path, ";", sub_path, NULL);
        }
        else {
          path = sub_path;
        }
    }
    
    /* We used ap_pstrdup because path is in temp_pool! */
    scfg->path = ap_pstrdup(parms->pool, path);
    modlua_lua_setluapath(path);
    return NULL;
}

const char *modlua_cmd_require(cmd_parms *parms, void *mconfig,
                               const char *arg)
{ 
    modlua_lua_init(parms->pool);
    return modlua_lua_require(parms->temp_pool, arg);
}

const char *modlua_cmd_handler(cmd_parms *parms, void *mconfig,
                               const char *arg)
{
    int event;
    if (!modlua_lua_isfuncname(arg))
        return "Lua handler name is invalid";
    event = (int)parms->info;
    modlua_config_dir_t *dcfg = (modlua_config_dir_t *)mconfig;
    char **new = (char **)ap_push_array(dcfg->handlers[event]);
    *new = ap_pstrdup(parms->pool, arg);
    return NULL;
}
