/*
** $Id: modlua_handler.c,v 1.1.1.1 2003/06/30 19:18:24 michel Exp $
** mod_lua - Apache module for Lua language
** See copyright notice in distro's COPYRIGHT file
*/

#include <httpd.h>
#include <http_log.h>

#include "modlua_handler.h"
#include "modlua_config.h"
#include "modlua_lua.h"
#include "mod_lua.h"

static int modlua_handler(request_rec *r, int event)
{
    modlua_config_dir_t *dcfg = (modlua_config_dir_t *)
        ap_get_module_config(r->per_dir_config, &lua_module);
    char **funcnames = (char **)dcfg->handlers[event]->elts;
    int n = dcfg->handlers[event]->nelts;
    int status = DECLINED;
    int i;

    for (i = 0; i < n; i++) {
        char *funcname = funcnames[i];
        const char *errmsg;

        status = modlua_lua_call_handler(r, funcname, &errmsg);
        
        if (*errmsg) {
            ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
                "Fail on call '%s': %s.", funcname, errmsg);
        }
                
        if (status != OK && status != DECLINED && status != DONE) {
            /* Other status abort the loop. */
            break;        
        }
    }

    return status;
}

int modlua_handler_postreadrequest(request_rec *r)
{
    return modlua_handler(r, EVENT_POSTREADREQUEST);  
}

int modlua_handler_trans(request_rec *r)
{
    return modlua_handler(r, EVENT_TRANS);  
}

int modlua_handler_headerparser(request_rec *r)
{
    return modlua_handler(r, EVENT_HEADERPARSER);  
}

int modlua_handler_access(request_rec *r)
{
    return modlua_handler(r, EVENT_ACCESS);  
}

int modlua_handler_authen(request_rec *r)
{
    return modlua_handler(r, EVENT_AUTHEN);  
}

int modlua_handler_authz(request_rec *r)
{
    return modlua_handler(r, EVENT_AUTHZ);  
}

int modlua_handler_type(request_rec *r)
{
    return modlua_handler(r, EVENT_TYPE);  
}

int modlua_handler_fixup(request_rec *r)
{
    return modlua_handler(r, EVENT_FIXUP);  
}

int modlua_handler_response(request_rec *r)
{
    return modlua_handler(r, EVENT_RESPONSE);  
}

int modlua_handler_log(request_rec *r)
{
    return modlua_handler(r, EVENT_LOG);  
}

